<!--
// © University of Southampton IT Innovation Centre, 2017
//
// Copyright in this software belongs to University of Southampton
// IT Innovation Centre of Gamma House, Enterprise Road, 
// Chilworth Science Park, Southampton, SO16 7NS, UK.
//
// This software may not be used, sold, licensed, transferred, copied
// or reproduced in whole or in part in any manner or form or in or
// on any media by any person other than in accordance with the terms
// of the Licence Agreement supplied with the software, or otherwise
// without the prior written consent of the copyright owners.
//
// This software is distributed WITHOUT ANY WARRANTY, without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE, except where stated in the Licence Agreement supplied with
// the software.
//
//      Created By :            Michael Boniface
//      Created Date :          18-12-2017
//      Created for Project :   FLAME
-->

# FLAME Cross Layer Management and Control
#### Version: 1.0.0

### About this document

#### Authors

|Authors|Organisation|                    
|-|-|
|[Michael Boniface](mailto:mjb@it-innovation.soton.ac.uk)|[University of Southampton, IT Innovation Centre](http://www.it-innovation.soton.ac.uk)|
|[Simon Crowle](mailto:sgc@it-innovation.soton.ac.uk)|[University of Southampton, IT Innovation Centre](http://www.it-innovation.soton.ac.uk)|

### Contents

#### Documentation

Implementation documentation and discussion can be found in the [FLAME CLMC Information Model Specification](docs/clmc-information-model.md)

#### Development Environment

The development environment is currently Vagrant/Virtualbox providing the infrastructure for dev and testing.

Vagrant-disksize plugin must be installed. 

`vagrant plugin install vagrant-disksize`

#### Testing

Testing is implemented using pytest using the following convention:

* Tests are written in python using pytest
* Related tests are stored in a python module `clmctest/<testmodule>` to create a suite of tests. All tests are stored in files test_*.py, there can be many tests per file, and many files per module
* Each test module has a rspec.yml that provides the baseline "fixture" for the tests in the module
* Tests are executed against fixtures. Fixtures are modular "setups" created for a test, that are inserted into the python code using dependancy injection. This offers more flexibility than the unit style testing. The baseline deployment is created using `vagrant up` with an appropriate rspec, and the pytest fixture reads the rspec.yml and makes the configuration available to the test.
* Tests are executed from a guest VM (not the host) in the repo root using the command `pytest test/<testmodule>`
* Pytest will scan the directory for all tests including in files test_*.py and run them

#### Creating a deployment for a test

Each test has a fixture defining a set of VMs required for the test. The fixture is described in an rspec.yml file that is stored within the clmctest/<testmoduel> directory. For example, the monitoring test fixture is stored in the following file.

`clmctest\monitoring\rspec.yml`

The fixture is created by running the vagrant up command with the test module name as a parameter

`vagrant --fixture=monitoring -- up`

This will provision the following VMs clmc-service, ipendpoint1, ipendpoint2

The **clmc-service** vm includes influx, Kapacitor and Chronograf. The following ports forwarded to the clmc VM from the host machine are as follows:

* Influx: 8086 
* Chronograf: 8888
* Kapacitor: 9092

Typically, tests are run from a test-runner VM that has pytest installed. The script used to install pytest is 

`clmctest/services/pytest/install.sh`

To run a test, SSH into the test runner machine

`vagrant --fixture=monitoring -- ssh test-runner`

Then go to the 'vagrant' directory.

`cd /vagrant`

Run the pytest.

`pytest -s clmctest/monitoring/`

#### Tests run from the host

Tests can be run from the host by sshing commands to the test running. This is described in the .gitlab-ci.yml file. The file shows how a python package is created

`python setup.py sdist --dist-dir=build`

Then the package is installed 

`vagrant --fixture=scripts -- ssh test-runner -- -tt "pip3 install /vagrant/build/clmctest-SNAPSHOT.tar.gz"`

Then the tests are run

`sudo apt-get install python3-pip` 
  
`pip3 install pytest`


#### CLMC Service

The CLMC service is implemented using the Pyramid framework. (currently under development)

Before installing the CLMC service and its dependencies, it is recommended to use a virtual environment. To manage virtual
environments, **virtualenvwrapper** can be used.

```
pip install virtualenvwrapper
```

To create a virtual environment use the **mkvirtualenv** command:

```
mkvirtualenv CLMC
```

When created, you should already be set to use the new virtual environment, but to make sure of this use the **workon** command:

```
workon CLMC
```

Now, any installed libraries will be specificly installed in this environment only. To install and use the CLMC service 
locally, the easiest thing to do is to use **pip** (make sure you are in the root folder of the project - ***flame-clmc***):

```
pip install -e .
```

Finally, start the service on localhost by using pyramid's **pserve**:

```
pserve development.ini --reload
```

You should now be able to see the 'Hello world' message when visiting **http://localhost:8080** in your browser.
